@@ -36,10 +36,11 @@ macro_rules! helper_expr {
         let external = $crate::helpers::HELPERS.with(|helper| helper.external());
 
         if external {
-            quote_ident!(span, "swcHelpers")
-                .make_member(quote_ident!($span, $crate::external_name!($s)))
+            swc_ecma_utils::quote_ident!(span, "swcHelpers").make_member(
+                swc_ecma_utils::quote_ident!($span, $crate::external_name!($s)),
+            )
         } else {
-            Expr::from(quote_ident!(span, concat!('_', $s)))
+            Expr::from(swc_ecma_utils::quote_ident!(span, concat!('_', $s)))
         }
     }};
 }
